# This sample will log in using the given JID (1st arg)
# it will then join the given MUC room (2nd arg)
# at the given muc service (3rd arg)
# using the given nick name (4th arg)
# it will print out status changes of the room members as well as any messages.
# to send a message to the room just the type the message text, hit enter.
# that's it.
#
# python mucclient.py yourjid@domain.tld room yourroomnick
#

import gloox
import sys
import select
import getpass

class MyClient( gloox.ConnectionListener, gloox.MUCRoomHandler ):
  j = gloox.JID( sys.argv[1] )
  c = gloox.Client( j, '' )
  m_conn = gloox.ConnectionTCPClient( c, c.logInstance(), j.server() )
  c.setConnectionImpl( m_conn );
  c.disco().setIdentity( 'client', 'console' )
  c.disco().setVersion( 'mucclient based on pygloox', '0.9', 'linux' )
  gloox_fd = 0
  input_fd = sys.stdin.fileno()
  b = False
  nick = gloox.JID( sys.argv[2] + '@' + sys.argv[3] + '/' + sys.argv[4] )
  m_room = gloox.MUCRoom( c, nick, None )


  def __init__( self ):
    gloox.ConnectionListener.__init__( self )
    gloox.MUCRoomHandler.__init__( self )

  def setPassword( self, pwd ):
    self.c.setPassword( pwd )

  def connect( self ):
    self.c.registerConnectionListener( self )
    self.c.setPresence( gloox.PresenceAvailable, -1, '' )
    self.b = self.c.connect( False )
    self.gloox_fd = self.m_conn.socket()

  def handleMUCParticipantPresence( self, room, participant, presence ):
    if presence == gloox.PresenceAvailable:
      print participant.nick.resource() + ' is in the room'
    elif presence == gloox.PresenceUnavailable:
      print participant.nick.resource() + ' left the room'
    elif presence == gloox.PresenceAway:
      print participant.nick.resource() + ' is now Away'
    elif presence == gloox.PresenceDnd:
      print participant.nick.resource() + ' is now DnD'
    elif presence == gloox.PresenceXa:
      print participant.nick.resource() + ' is now XA'
    elif presence == gloox.PresenceChat:
      print participant.nick.resource() + ' is now available for chat'

  def handleMUCMessage( self, room, nick, message, history, when, privateMessage ):
    if history == True:
      print nick + ': ' + message + ' (history)'
    elif privateMessage == True:
      print nick + ': ' + message + ' (private)'
    else:
      print nick + ': ' + message

  def handleMUCRoomCreation( self, room ):
    print 'room created. uhoh.'

  def handleMUCSubject( self, room, nick, subject ):
    print 'new subject: ' + subject

  def handleMUCInviteDecline( self, room, invitee, reason ):
    print 'invitation declined'

  def handleMUCError( self, room, error ):
    print 'error'

  def handleMUCInfo( self, room, features, name, infoForm ):
    print 'info...'

  def handleMUCItems( self, room, items ):
    print 'items...'

  def onConnect( self ):
    self.m_room.registerMUCRoomHandler( self )
    print 'joining ' + sys.argv[2]
    self.m_room.join();

  def onTLSConnect( self, info ):
    return True

  def onDisconnect( self, e ):
    print 'disconnected'

  def run( self ):
    fds = [self.gloox_fd, self.input_fd]
    if self.b:
      print 'connection established'
      while self.b:
        r = select.select( fds, [], [], 0.01 )
        for fd in r[0]:
          if fd == self.gloox_fd:
            self.c.recv( -1 )
          else:
            txt = sys.stdin.readline()
            if txt[0:-1] == "!away":
              self.m_room.setPresence( gloox.PresenceAway )
            elif txt[0:-1] == "!xa":
              self.m_room.setPresence( gloox.PresenceXa )
            elif txt[0:-1] == "!available":
              self.m_room.setPresence( gloox.PresenceAvailable )
            elif txt[0:5] == "!nick":
              self.m_room.setNick( txt[6:-1] )
            elif txt[0:-1] == "!quit":
              self.m_room.leave()
              self.c.disconnect()
            elif txt[0:-1] == ":q":
              self.m_room.leave()
              self.c.disconnect()
            elif txt[0:3] == "!py":
              try:
                print 'execing: ' + txt[4:-1]
                exec txt[4:-1]
              except:
                print 'didn\'t work, exception thrown'
            else:
              self.m_room.send( txt[:-1] )

t = MyClient()
pwd = getpass.getpass( 'Password for ' + sys.argv[1] + ': ' )
t.setPassword( pwd )
t.connect()
t.run()
